from os.path import dirname, abspath, join as pjoin
from werkzeug import run_simple, Request, Response
from werkzeug.routing import Map, Rule
from werkzeug.exceptions import HTTPException
from genshi.template import TemplateLoader
from flatland.out.genshi import flatland_filter


template_loader = TemplateLoader(
    abspath(pjoin(dirname(__file__), '..', 'templates')),
    callback=lambda t: t.filters.append(flatland_filter),
    auto_reload=True)

url_map = Map()

def render(file, **context):
    template = template_loader.load(file)
    routes = sorted(r.rule for r in url_map.iter_rules())
    context.update(routes=routes)
    return template.generate(**context).render()

def web_action(rule):
    def decorate(fn):
        def web_app(req):
            return Response(fn(req), content_type='text/html')
        web_app.__name__ = fn.__name__
        kw = dict(endpoint=web_app)
        url_map.add(Rule(rule, **kw))
        return web_app
    return decorate

class DemoRequest(Request):
    post_data = Request.form

def wsgi(environ, start_response):
    request = DemoRequest(environ)
    adapter = url_map.bind_to_environ(environ)
    try:
        endpoint, values = adapter.match()
        response = endpoint(request)
    except HTTPException, exc:
        response = exc
    return response(environ, start_response)

def run():
    run_simple('localhost', 8001, wsgi)
